@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #ifndef MEDIASTREAM_H
@@ -23,6 +23,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include <ortp/ortp.h>
 #include <ortp/event.h>
+#include <ortp/nack.h>
 
 #include <mediastreamer2/msfactory.h>
 #include <mediastreamer2/msfilter.h>
@@ -30,6 +31,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <mediastreamer2/mssndcard.h>
 #include <mediastreamer2/mswebcam.h>
 #include <mediastreamer2/msvideo.h>
+#include <mediastreamer2/msvideoqualitycontroller.h>
 #include <mediastreamer2/bitratecontrol.h>
 #include <mediastreamer2/qualityindicator.h>
 #include <mediastreamer2/ice.h>
@@ -49,6 +51,7 @@ extern "C" {
 
 struct _RingStream
 {
+	MSSndCard *card;
 	MSTicker *ticker;
 	MSFilter *source;
 	MSFilter *gendtmf;
@@ -136,14 +139,17 @@ struct _MediaStream {
 	 * defines encoder target network bit rate, uses #media_stream_set_target_network_bitrate() setter.
 	 * */
 	int target_bitrate;
+	int max_target_bitrate;
 	media_stream_process_rtcp_callback_t process_rtcp;
 	OrtpEvDispatcher *evd;
 	MSFactory *factory;
 	MSBandwidthController *bandwidth_controller;
+	MSVideoQualityController *video_quality_controller;
 };
 
 MS2_PUBLIC void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStreamSessions *sessions);
 
+MS2_PUBLIC MSFactory* media_stream_get_factory(MediaStream *stream);
 
 /**
  * @addtogroup audio_stream_api
@@ -157,7 +163,7 @@ MS2_PUBLIC int media_stream_join_multicast_group(MediaStream *stream, const char
 MS2_PUBLIC bool_t media_stream_dtls_supported(void);
 
 /* enable DTLS on the media stream */
-MS2_PUBLIC void media_stream_enable_dtls(MediaStream *stream, MSDtlsSrtpParams *params);
+MS2_PUBLIC void media_stream_enable_dtls(MediaStream *stream, const MSDtlsSrtpParams *params);
 
 MS2_PUBLIC void media_stream_set_rtcp_information(MediaStream *stream, const char *cname, const char *tool);
 
@@ -228,6 +234,15 @@ MS2_PUBLIC float media_stream_get_average_lq_quality_rating(MediaStream *stream)
  * */
 MS2_PUBLIC int media_stream_set_target_network_bitrate(MediaStream *stream,int target_bitrate);
 
+/**
+ * Set a maximum target bitrate for the stream. Indeed, the MSBandwidthController may adapt the target bitrate
+ * according to network conditions, which includes the possibility to increase it if remote side sends a TMMBR
+ * to invite to increase bitrate. The max_network_bitrate defines the upper limit for increasing the bitrate usage automatically.
+ * @param stream stream to apply parameter on
+ * @param target_bitrate in bit per seconds
+**/
+MS2_PUBLIC int media_stream_set_max_network_bitrate(MediaStream *stream,int max_bitrate);
+
 /**
  * get the stream target bitrate.
  * @param stream stream to apply parameter on
@@ -331,7 +346,7 @@ MS2_PUBLIC bool_t ms_media_resource_is_consistent(const MSMediaResource *r);
 #define ms_media_resource_get_file(r)		(((r)->type == MSResourceFile) ? (r)->file : NULL)
 #define ms_media_resource_get_rtp_session(r)	(((r)->type == MSResourceRtp) ? (r)->session : NULL)
 #define ms_media_resource_get_camera(r)		(((r)->type == MSResourceCamera) ? (r)->camera : NULL)
-#define ms_media_resource_get_soundcard(r)	(((r)->type == MSResourceSoundcard) ? (r)->souncard : NULL)
+#define ms_media_resource_get_soundcard(r)	(((r)->type == MSResourceSoundcard) ? (r)->soundcard : NULL)
 /**
  * Structure describing the input/output of a MediaStream.
  * Input and output are described as MSMediaResource.
@@ -341,7 +356,7 @@ typedef struct _MSMediaStreamIO {
 	MSMediaResource output;
 } MSMediaStreamIO;
 
-#define MS_MEDIA_STREAM_IO_INITIALIZER { {0}, {0} }
+#define MS_MEDIA_STREAM_IO_INITIALIZER { {MSResourceInvalid}, {MSResourceInvalid} }
 
 MS2_PUBLIC bool_t ms_media_stream_io_is_consistent(const MSMediaStreamIO *io);
 
@@ -384,6 +399,7 @@ struct _AudioStream
 		int videopin;
 		bool_t plumbed;
 	}av_player;
+	MSFilter *flowcontrol;
 	RtpSession *rtp_io_session; /**< The RTP session used for RTP input/output. */
 	MSFilter *vaddtx;
 	char *recorder_file;
@@ -398,7 +414,7 @@ struct _AudioStream
 	bool_t use_ec;
 	bool_t use_gc;
 	bool_t use_agc;
-	
+
 	bool_t mic_eq_active;
 	bool_t spk_eq_active;
 	bool_t use_ng;/*noise gate*/
@@ -479,7 +495,7 @@ MS2_PUBLIC void audio_stream_play_received_dtmfs(AudioStream *st, bool_t yesno);
  * @param loc_rtp_port the local UDP port to listen for RTP packets.
  * @param loc_rtcp_port the local UDP port to listen for RTCP packets
  * @param ipv6 TRUE if ipv6 must be used.
- * @param factory 
+ * @param factory
  * @return a new AudioStream.
 **/
 MS2_PUBLIC AudioStream *audio_stream_new(MSFactory* factory, int loc_rtp_port, int loc_rtcp_port, bool_t ipv6);
@@ -512,6 +528,7 @@ MS2_PUBLIC AudioStream *audio_stream_new_with_sessions(MSFactory* factory, const
 #define AUDIO_STREAM_FEATURE_MIXED_RECORDING	(1 << 7)
 #define AUDIO_STREAM_FEATURE_LOCAL_PLAYING	(1 << 8)
 #define AUDIO_STREAM_FEATURE_REMOTE_PLAYING	(1 << 9)
+#define AUDIO_STREAM_FEATURE_FLOW_CONTROL	(1 << 10)
 
 #define AUDIO_STREAM_FEATURE_ALL	(\
 					AUDIO_STREAM_FEATURE_PLC | \
@@ -523,7 +540,8 @@ MS2_PUBLIC AudioStream *audio_stream_new_with_sessions(MSFactory* factory, const
 					AUDIO_STREAM_FEATURE_DTMF_ECHO |\
 					AUDIO_STREAM_FEATURE_MIXED_RECORDING |\
 					AUDIO_STREAM_FEATURE_LOCAL_PLAYING | \
-					AUDIO_STREAM_FEATURE_REMOTE_PLAYING \
+					AUDIO_STREAM_FEATURE_REMOTE_PLAYING | \
+					AUDIO_STREAM_FEATURE_FLOW_CONTROL \
 					)
 
 
@@ -606,7 +624,7 @@ static MS2_INLINE void audio_stream_enable_adaptive_jittcomp(AudioStream *stream
  * of the sound card nor the system mixer one. If you intends
  * to control the volume gain at sound card level, you should
  * use audio_stream_set_sound_card_input_gain() instead.
- * 
+ *
  * @param stream The stream.
  * @param gain_db Gain to apply in dB.
  */
@@ -616,9 +634,9 @@ MS2_PUBLIC void audio_stream_set_mic_gain_db(AudioStream *stream, float gain_db)
 /**
  * Like audio_stream_set_mic_gain_db() excepted that the gain is specified
  * in percentage.
- * 
+ *
  * @param stream The stream.
- * @param gain Gain to apply in percetage of the max supported gain.
+ * @param gain Gain to apply in percentage of the max supported gain.
  */
 MS2_PUBLIC void audio_stream_set_mic_gain(AudioStream *stream, float gain);
 
@@ -634,12 +652,21 @@ MS2_PUBLIC void audio_stream_mute_rtp(AudioStream *stream, bool_t val);
  */
 MS2_PUBLIC void audio_stream_set_spk_gain_db(AudioStream *stream, float gain_db);
 
+/**
+ * Like audio_stream_set_spk_gain_db() excepted that the gain is specified
+ * in percentage.
+ *
+ * @param stream The stream.
+ * @param gain Gain to apply in percentage of the max supported gain.
+ */
+MS2_PUBLIC void audio_stream_set_spk_gain(AudioStream *stream, float gain);
+
 /**
  * Set microphone volume gain.
  * If the sound backend supports it, the set volume gain will be synchronized
  * with the host system mixer. If you intended to apply a static software gain,
  * you should use audio_stream_set_mic_gain_db() or audio_stream_set_mic_gain().
- * 
+ *
  * @param stream The audio stream.
  * @param gain Percentage of the max supported volume gain. Valid values are in [0.0 : 1.0].
  */
@@ -777,6 +804,7 @@ static MS2_INLINE RtpSession * audio_stream_get_rtp_session(const AudioStream *s
 
 typedef void (*VideoStreamRenderCallback)(void *user_pointer, const MSPicture *local_view, const MSPicture *remote_view);
 typedef void (*VideoStreamEventCallback)(void *user_pointer, const MSFilter *f, const unsigned int event_id, const void *args);
+typedef void (*VideoStreamCameraNotWorkingCallback)(void *user_pointer, const MSWebCam *old_webcam);
 
 struct _MediastreamVideoStat
 {
@@ -791,22 +819,24 @@ typedef struct _MediastreamVideoStat MediaStreamVideoStat;
 struct _VideoStream
 {
 	MediaStream ms;
-	MSFilter *void_source;
-	MSFilter *source;
-	MSFilter *pixconv;
-	MSFilter *sizeconv;
-	MSFilter *tee;
-	MSFilter *output;
-	MSFilter *tee2;
 	MSFilter *jpegwriter;
-	MSFilter *output2;
-	MSFilter *tee3;
-	MSFilter *recorder_output; /*can be an ItcSink to send video to the audiostream's multimedia recorder, or directly a MkvRecorder */
 	MSFilter *local_jpegwriter;
+	MSFilter *output;
+	MSFilter *output2;
+	MSFilter *pixconv;
+	MSFilter *qrcode;
+	MSFilter *recorder_output; /*can be an ItcSink to send video to the audiostream's multimedia recorder, or directly a MkvRecorder */
+	MSFilter *sizeconv;
+	MSFilter *source;
+	MSFilter *tee;
+	MSFilter *tee2;
+	MSFilter *tee3;
+	MSFilter *void_source;
 	MSVideoSize sent_vsize;
 	MSVideoSize preview_vsize;
-	float fps; /*the target fps explicitely set by application, overrides internally selected fps*/
+	float forced_fps; /*the target fps explicitely set by application, overrides internally selected fps*/
 	float configured_fps; /*the fps that was configured to the encoder. It might be different from the one really obtained from camera.*/
+	float real_fps; /*the fps obtained from camera.*/
 	int corner; /*for selfview*/
 	VideoStreamRenderCallback rendercb;
 	void *render_pointer;
@@ -816,28 +846,36 @@ struct _VideoStream
 	void *window_id;
 	void *preview_window_id;
 	MediaStreamDir dir;
+	MSRect decode_rect; //Used for the qrcode decoder
 	MSWebCam *cam;
 	RtpSession *rtp_io_session; /**< The RTP session used for RTP input/output. */
 	char *preset;
+	MSVideoConfiguration *vconf_list;
+	struct _AudioStream *audiostream;/*the audio stream with which this videostream is paired*/
+	OrtpNackContext *nack_context;
 	int device_orientation; /* warning: meaning of this variable depends on the platform (Android, iOS, ...) */
 	uint64_t last_reported_decoding_error_time;
 	uint64_t last_fps_check;
+	uint64_t last_camera_check;
+	int dead_camera_check_count;
+	VideoStreamCameraNotWorkingCallback cameracb;
+	void *camera_pointer;
 	MediaStreamVideoStat ms_video_stat;
 	bool_t use_preview_window;
+	bool_t enable_qrcode_decoder;
 	bool_t freeze_on_error;
 	bool_t display_filter_auto_rotate_enabled;
-	bool_t source_performs_encoding;
 	
+	bool_t source_performs_encoding;
 	bool_t output_performs_decoding;
 	bool_t player_active;
 	bool_t staticimage_webcam_fps_optimization; /* if TRUE, the StaticImage webcam will ignore the fps target in order to save CPU time. Default is TRUE */
-	
 };
 
 typedef struct _VideoStream VideoStream;
 
 
-    
+
 MS2_PUBLIC VideoStream *video_stream_new(MSFactory* factory, int loc_rtp_port, int loc_rtcp_port, bool_t use_ipv6);
 /**
  * Creates a VideoStream object listening on a RTP port for a dedicated address.
@@ -858,10 +896,11 @@ static MS2_INLINE void video_stream_enable_adaptive_jittcomp(VideoStream *stream
 }
 MS2_PUBLIC void video_stream_set_render_callback(VideoStream *s, VideoStreamRenderCallback cb, void *user_pointer);
 MS2_PUBLIC void video_stream_set_event_callback(VideoStream *s, VideoStreamEventCallback cb, void *user_pointer);
+MS2_PUBLIC void video_stream_set_camera_not_working_callback(VideoStream *s, VideoStreamCameraNotWorkingCallback cb, void *user_pointer);
 MS2_PUBLIC void video_stream_set_display_filter_name(VideoStream *s, const char *fname);
 MS2_PUBLIC int video_stream_start_with_source(VideoStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
 		const char *rem_rtcp_ip, int rem_rtcp_port, int payload, int jitt_comp, MSWebCam* cam, MSFilter* source);
-MS2_PUBLIC int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port, const char *rem_rtcp_ip, 
+MS2_PUBLIC int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port, const char *rem_rtcp_ip,
 				  int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *device);
 MS2_PUBLIC int video_stream_start_with_files(VideoStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
         const char *rem_rtcp_ip, int rem_rtcp_port, int payload_type, const char *play_file, const char *record_file);
@@ -903,6 +942,8 @@ MS2_PUBLIC MSFilter* video_stream_get_source_filter(const VideoStream* stream);
 
 MS2_PUBLIC void video_stream_change_camera(VideoStream *stream, MSWebCam *cam);
 
+MS2_PUBLIC void video_stream_change_camera_skip_bitrate(VideoStream *stream, MSWebCam *cam);
+
 /**
  * @brief This functions changes the source filter for the passed video stream.
  * @details This is quite the same function as \ref video_stream_change_camera, but this one
@@ -1019,11 +1060,6 @@ MS2_PUBLIC float video_stream_get_sent_framerate(const VideoStream *stream);
  */
 MS2_PUBLIC float video_stream_get_received_framerate(const VideoStream *stream);
 
-/**
- * Returns the name of the video display filter on the current platform.
-**/
-MS2_PUBLIC const char *video_stream_get_default_video_renderer(void);
-
 MS2_PUBLIC void video_stream_enable_self_view(VideoStream *stream, bool_t val);
 MS2_PUBLIC void * video_stream_get_native_window_id(VideoStream *stream);
 MS2_PUBLIC void video_stream_set_native_window_id(VideoStream *stream, void *id);
@@ -1137,6 +1173,12 @@ MS2_PUBLIC void audio_stream_unlink_video(AudioStream *stream, VideoStream *vide
  */
 MS2_PUBLIC void video_stream_use_video_preset(VideoStream *stream, const char *preset);
 
+/**
+ * Returns the name of the video preset used for the video stream.
+ * @param[in] stream VideoStream object
+ */
+MS2_PUBLIC const char* video_stream_get_video_preset(VideoStream *stream);
+
 
 /**
  * Open a player to play a video file (mkv) to remote end.
@@ -1154,6 +1196,9 @@ MS2_PUBLIC MSFilter * video_stream_open_remote_record(VideoStream *stream, const
 
 MS2_PUBLIC void video_stream_close_remote_record(VideoStream *stream);
 
+MS2_PUBLIC void video_stream_enable_retransmission_on_nack(VideoStream *stream, bool_t enable);
+MS2_PUBLIC void video_stream_set_retransmission_on_nack_max_packet(VideoStream *stream, unsigned int max);
+
 /**
  * Small API to display a local preview window.
 **/
@@ -1161,13 +1206,17 @@ MS2_PUBLIC void video_stream_close_remote_record(VideoStream *stream);
 typedef VideoStream VideoPreview;
 
 MS2_PUBLIC VideoPreview * video_preview_new(MSFactory *factory);
-#define video_preview_set_size(p,s)			video_stream_set_sent_video_size(p,s)
-#define video_preview_set_display_filter_name(p,dt)	video_stream_set_display_filter_name(p,dt)
-#define video_preview_set_native_window_id(p,id)	video_stream_set_native_preview_window_id(p,id)
-#define video_preview_get_native_window_id(p)		video_stream_get_native_preview_window_id(p)
-#define video_preview_set_fps(p,fps)			video_stream_set_fps((VideoStream*)p,fps)
+#define video_preview_set_event_callback(p,c,u) video_stream_set_event_callback(p,c,u)
+#define video_preview_set_size(p,s) video_stream_set_sent_video_size(p,s)
+#define video_preview_set_display_filter_name(p,dt) video_stream_set_display_filter_name(p,dt)
+#define video_preview_set_native_window_id(p,id) video_stream_set_native_preview_window_id(p,id)
+#define video_preview_get_native_window_id(p) video_stream_get_native_preview_window_id(p)
+#define video_preview_set_fps(p,fps) video_stream_set_fps((VideoStream*)p,fps)
 #define video_preview_set_device_rotation(p, r) video_stream_set_device_rotation(p, r)
 MS2_PUBLIC void video_preview_start(VideoPreview *stream, MSWebCam *device);
+MS2_PUBLIC void video_preview_enable_qrcode(VideoPreview *stream, bool_t enable);
+MS2_PUBLIC void video_preview_set_decode_rect(VideoPreview *stream, MSRect rect);
+MS2_PUBLIC bool_t video_preview_qrcode_enabled(VideoPreview *stream);
 MS2_PUBLIC MSVideoSize video_preview_get_current_size(VideoPreview *stream);
 MS2_PUBLIC void video_preview_stop(VideoPreview *stream);
 MS2_PUBLIC void video_preview_change_camera(VideoPreview *stream, MSWebCam *cam);
@@ -1189,7 +1238,6 @@ MS2_PUBLIC MSFilter* video_preview_stop_reuse_source(VideoPreview *stream);
 **/
 MS2_PUBLIC MSWebCamDesc *ms_mire_webcam_desc_get(void);
 
-
 /**
  * Create an RTP session for duplex communication.
  * @param[in] local_ip The local IP to bind the RTP and RTCP sockets to.
@@ -1230,7 +1278,7 @@ typedef struct _TextStream TextStream;
  * @param loc_rtp_port the local UDP port to listen for RTP packets.
  * @param loc_rtcp_port the local UDP port to listen for RTCP packets
  * @param ipv6 TRUE if ipv6 must be used.
- * @param factory 
+ * @param factory
  * @return a new TextStream.
 **/
 MS2_PUBLIC TextStream *text_stream_new(MSFactory *factory, int loc_rtp_port, int loc_rtcp_port, bool_t ipv6);
@@ -1248,7 +1296,7 @@ MS2_PUBLIC TextStream *text_stream_new_with_sessions(MSFactory *factory, const M
  * @param loc_ip the local ip to listen for RTP packets. Can be ::, O.O.O.O or any ip4/6 addresses
  * @param [in] loc_rtp_port the local UDP port to listen for RTP packets.
  * @param [in] loc_rtcp_port the local UDP port to listen for RTCP packets
- * @param factory 
+ * @param factory
  * @return a new TextStream.
 **/
 MS2_PUBLIC TextStream *text_stream_new2(MSFactory *factory, const char* ip, int loc_rtp_port, int loc_rtcp_port);
@@ -1276,14 +1324,14 @@ MS2_PUBLIC void text_stream_stop (TextStream * stream);
 /**
  * Executes background low priority tasks related to text processing (RTP statistics analysis).
  * It should be called periodically, for example with an interval of 100 ms or so.
- * 
+ *
  * @param[in] stream TextStream object previously created with text_stream_new().
  */
 MS2_PUBLIC void text_stream_iterate(TextStream *stream);
 
 /**
  * Writes a character to stream in UTF-32 format.
- * 
+ *
  * @param[in] stream TextStream object previously created with text_stream_new().
  * @param[in] i the Char in UTF-32 format.
  **/
@@ -1296,6 +1344,7 @@ MS2_PUBLIC void text_stream_unprepare_text(TextStream *stream);
  * @}
 **/
 
+
 #ifdef __cplusplus
 }
 #endif
